@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2013-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2013-2025 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -323,17 +323,20 @@ static CPUMMSRRANGE const g_aMsrRanges_Intel_Core_i5_3570[] =
 /**
  * Database entry for Intel(R) Core(TM) i5-3570 CPU @ 3.40GHz.
  */
-static CPUMDBENTRY const g_Entry_Intel_Core_i5_3570 =
+static CPUMDBENTRYX86 const g_Entry_Intel_Core_i5_3570 =
 {
-    /*.pszName          = */ "Intel Core i5-3570",
-    /*.pszFullName      = */ "Intel(R) Core(TM) i5-3570 CPU @ 3.40GHz",
-    /*.enmVendor        = */ CPUMCPUVENDOR_INTEL,
+    {
+        /*.pszName      = */ "Intel Core i5-3570",
+        /*.pszFullName  = */ "Intel(R) Core(TM) i5-3570 CPU @ 3.40GHz",
+        /*.enmVendor    = */ CPUMCPUVENDOR_INTEL,
+        /*.enmMicroarch = */ kCpumMicroarch_Intel_Core7_IvyBridge,
+        /*.fFlags       = */ 0,
+        /*.enmEntryType = */ CPUMDBENTRYTYPE_X86,
+    },
     /*.uFamily          = */ 6,
     /*.uModel           = */ 58,
     /*.uStepping        = */ 9,
-    /*.enmMicroarch     = */ kCpumMicroarch_Intel_Core7_IvyBridge,
     /*.uScalableBusFreq = */ CPUM_SBUSFREQ_100MHZ,
-    /*.fFlags           = */ 0,
     /*.cMaxPhysAddrWidth= */ 36,
     /*.fMxCsrMask       = */ 0xffff,
     /*.paCpuIdLeaves    = */ NULL_ALONE(g_aCpuIdLeaves_Intel_Core_i5_3570),
